@@ -36,6 +36,7 @@
 #include "qemu_domain.h"
 #include "qemu_firmware.h"
 #include "qemu_namespace.h"
+#include "qemu_saveimage.h"
 #include "qemu_security.h"
 #include "viruuid.h"
 #include "virconf.h"
@@ -64,6 +65,9 @@ VIR_LOG_INIT("qemu.qemu_conf");
 #define QEMU_REMOTE_PORT_MIN 5900
 #define QEMU_REMOTE_PORT_MAX 65535
 
+#define QEMU_RDP_PORT_MIN 3389
+#define QEMU_RDP_PORT_MAX 65535
+
 #define QEMU_WEBSOCKET_PORT_MIN 5700
 #define QEMU_WEBSOCKET_PORT_MAX 65535
 
@@ -103,6 +107,7 @@ VIR_ONCE_GLOBAL_INIT(virQEMUConfig);
 
 #define QEMU_BRIDGE_HELPER "qemu-bridge-helper"
 #define QEMU_PR_HELPER "qemu-pr-helper"
+#define QEMU_RDP "qemu-rdp"
 #define QEMU_DBUS_DAEMON "dbus-daemon"
 
 
@@ -225,6 +230,7 @@ virQEMUDriverConfig *virQEMUDriverConfigNew(bool privileged,
     cfg->configDir = g_strdup_printf("%s/qemu", cfg->configBaseDir);
     cfg->autostartDir = g_strdup_printf("%s/qemu/autostart", cfg->configBaseDir);
     cfg->slirpStateDir = g_strdup_printf("%s/slirp", cfg->stateDir);
+    cfg->rdpStateDir = g_strdup_printf("%s/rdp", cfg->stateDir);
     cfg->passtStateDir = g_strdup_printf("%s/passt", cfg->stateDir);
     cfg->dbusStateDir = g_strdup_printf("%s/dbus", cfg->stateDir);
 
@@ -239,11 +245,15 @@ virQEMUDriverConfig *virQEMUDriverConfigNew(bool privileged,
     }
 
     cfg->vncListen = g_strdup(VIR_LOOPBACK_IPV4_ADDR);
+    cfg->rdpListen = g_strdup(VIR_LOOPBACK_IPV4_ADDR);
     cfg->spiceListen = g_strdup(VIR_LOOPBACK_IPV4_ADDR);
 
     cfg->remotePortMin = QEMU_REMOTE_PORT_MIN;
     cfg->remotePortMax = QEMU_REMOTE_PORT_MAX;
 
+    cfg->rdpPortMin = QEMU_RDP_PORT_MIN;
+    cfg->rdpPortMax = QEMU_RDP_PORT_MAX;
+
     cfg->webSocketPortMin = QEMU_WEBSOCKET_PORT_MIN;
     cfg->webSocketPortMax = QEMU_WEBSOCKET_PORT_MAX;
 
@@ -264,6 +274,7 @@ virQEMUDriverConfig *virQEMUDriverConfigNew(bool privileged,
     cfg->prHelperName = g_strdup(QEMU_PR_HELPER);
     cfg->slirpHelperName = g_strdup(QEMU_SLIRP_HELPER);
     cfg->dbusDaemonName = g_strdup(QEMU_DBUS_DAEMON);
+    cfg->qemuRdpName = g_strdup(QEMU_RDP);
 
     cfg->securityDefaultConfined = true;
     cfg->securityRequireConfined = false;
@@ -291,14 +302,34 @@ virQEMUDriverConfig *virQEMUDriverConfigNew(bool privileged,
     cfg->deprecationBehavior = g_strdup("none");
     cfg->storageUseNbdkit = USE_NBDKIT_DEFAULT;
 
-#ifndef MADV_DONTDUMP
+#ifdef MADV_DONTDUMP
+    cfg->maxCore = ULLONG_MAX;
+#else
     /* QEMU uses Linux extensions to madvise() (MADV_DODUMP/MADV_DONTDUMP) to
-     * include/exclude guest memory from core dump. These might be unavailable
-     * on some systems. Provide sane default. */
+     * include/exclude guest memory from core dump. On non-Linux systems
+     * core dumps will unavoidably include all guest RAM, so toggle the
+     * default to reflect this and warn the admin.
+     */
     VIR_INFO("Host kernel doesn't support MADV_DONTDUMP. Enabling dump_guest_core");
     cfg->dumpGuestCore = true;
 #endif
 
+    if (privileged) {
+        /*
+         * Defer to libvirt-guests.service.
+         *
+         * XXX, or query if libvirt-guests.service is enabled perhaps ?
+         */
+        cfg->autoShutdownTrySave = VIR_DOMAIN_DRIVER_AUTO_SHUTDOWN_SCOPE_NONE;
+        cfg->autoShutdownTryShutdown = VIR_DOMAIN_DRIVER_AUTO_SHUTDOWN_SCOPE_NONE;
+        cfg->autoShutdownPoweroff = VIR_DOMAIN_DRIVER_AUTO_SHUTDOWN_SCOPE_NONE;
+    } else {
+        cfg->autoShutdownTrySave = VIR_DOMAIN_DRIVER_AUTO_SHUTDOWN_SCOPE_PERSISTENT;
+        cfg->autoShutdownTryShutdown = VIR_DOMAIN_DRIVER_AUTO_SHUTDOWN_SCOPE_ALL;
+        cfg->autoShutdownPoweroff = VIR_DOMAIN_DRIVER_AUTO_SHUTDOWN_SCOPE_ALL;
+    }
+    cfg->autoShutdownRestore = true;
+
     return g_steal_pointer(&cfg);
 }
 
@@ -322,6 +353,7 @@ static void virQEMUDriverConfigDispose(void *obj)
     g_free(cfg->slirpStateDir);
     g_free(cfg->passtStateDir);
     g_free(cfg->dbusStateDir);
+    g_free(cfg->rdpStateDir);
 
     g_free(cfg->libDir);
     g_free(cfg->cacheDir);
@@ -345,12 +377,14 @@ static void virQEMUDriverConfigDispose(void *obj)
     g_free(cfg->spicePassword);
     g_free(cfg->spiceSASLdir);
 
+    g_free(cfg->rdpTLSx509certdir);
+    g_free(cfg->rdpListen);
+    g_free(cfg->rdpUsername);
+    g_free(cfg->rdpPassword);
+
     g_free(cfg->chardevTLSx509certdir);
     g_free(cfg->chardevTLSx509secretUUID);
 
-    g_free(cfg->vxhsTLSx509certdir);
-    g_free(cfg->vxhsTLSx509secretUUID);
-
     g_free(cfg->nbdTLSx509certdir);
     g_free(cfg->nbdTLSx509secretUUID);
 
@@ -369,10 +403,8 @@ static void virQEMUDriverConfigDispose(void *obj)
     g_free(cfg->prHelperName);
     g_free(cfg->slirpHelperName);
     g_free(cfg->dbusDaemonName);
+    g_free(cfg->qemuRdpName);
 
-    g_free(cfg->saveImageFormat);
-    g_free(cfg->dumpImageFormat);
-    g_free(cfg->snapshotImageFormat);
     g_free(cfg->autoDumpPath);
 
     g_strfreev(cfg->securityDriverNames);
@@ -386,6 +418,8 @@ static void virQEMUDriverConfigDispose(void *obj)
 
     g_strfreev(cfg->capabilityfilters);
 
+    g_strfreev(cfg->sharedFilesystems);
+
     g_free(cfg->deprecationBehavior);
 }
 
@@ -494,6 +528,21 @@ virQEMUDriverConfigLoadSPICEEntry(virQEMUDriverConfig *cfg,
     return 0;
 }
 
+static int
+virQEMUDriverConfigLoadRDPEntry(virQEMUDriverConfig *cfg,
+                                virConf *conf)
+{
+    if (virConfGetValueString(conf, "rdp_tls_x509_cert_dir", &cfg->rdpTLSx509certdir) < 0)
+        return -1;
+    if (virConfGetValueString(conf, "rdp_listen", &cfg->rdpListen) < 0)
+        return -1;
+    if (virConfGetValueString(conf, "rdp_username", &cfg->rdpUsername) < 0)
+        return -1;
+    if (virConfGetValueString(conf, "rdp_password", &cfg->rdpPassword) < 0)
+        return -1;
+
+    return 0;
+}
 
 static int
 virQEMUDriverConfigLoadSpecificTLSEntry(virQEMUDriverConfig *cfg,
@@ -501,8 +550,6 @@ virQEMUDriverConfigLoadSpecificTLSEntry(virQEMUDriverConfig *cfg,
 {
     int rv;
 
-    if (virConfGetValueBool(conf, "vxhs_tls", &cfg->vxhsTLS) < 0)
-        return -1;
     if (virConfGetValueBool(conf, "nbd_tls", &cfg->nbdTLS) < 0)
         return -1;
     if (virConfGetValueBool(conf, "chardev_tls", &cfg->chardevTLS) < 0)
@@ -539,8 +586,6 @@ virQEMUDriverConfigLoadSpecificTLSEntry(virQEMUDriverConfig *cfg,
     GET_CONFIG_TLS_CERTINFO_COMMON(backup);
     GET_CONFIG_TLS_CERTINFO_SERVER(backup);
 
-    GET_CONFIG_TLS_CERTINFO_COMMON(vxhs);
-
     GET_CONFIG_TLS_CERTINFO_COMMON(nbd);
 
 #undef GET_CONFIG_TLS_CERTINFO_COMMON
@@ -620,18 +665,106 @@ static int
 virQEMUDriverConfigLoadSaveEntry(virQEMUDriverConfig *cfg,
                                  virConf *conf)
 {
-    if (virConfGetValueString(conf, "save_image_format", &cfg->saveImageFormat) < 0)
+    g_autofree char *savestr = NULL;
+    g_autofree char *dumpstr = NULL;
+    g_autofree char *snapstr = NULL;
+    g_autofree char *autoShutdownTrySave = NULL;
+    g_autofree char *autoShutdownTryShutdown = NULL;
+    g_autofree char *autoShutdownPoweroff = NULL;
+    int autoShutdownVal;
+
+    if (virConfGetValueString(conf, "save_image_format", &savestr) < 0)
         return -1;
-    if (virConfGetValueString(conf, "dump_image_format", &cfg->dumpImageFormat) < 0)
+    if (savestr && (cfg->saveImageFormat = qemuSaveFormatTypeFromString(savestr)) < 0) {
+        virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
+                       _("Invalid save_image_format '%1$s'"),
+                       savestr);
         return -1;
-    if (virConfGetValueString(conf, "snapshot_image_format", &cfg->snapshotImageFormat) < 0)
+    }
+
+    if (virConfGetValueString(conf, "dump_image_format", &dumpstr) < 0)
         return -1;
+    if (dumpstr && (cfg->dumpImageFormat = qemuSaveFormatTypeFromString(dumpstr)) < 0) {
+        virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
+                       _("Invalid dump_image_format '%1$s'"),
+                       dumpstr);
+        return -1;
+    }
+
+    if (virConfGetValueString(conf, "snapshot_image_format", &snapstr) < 0)
+        return -1;
+    if (snapstr && (cfg->snapshotImageFormat = qemuSaveFormatTypeFromString(snapstr)) < 0) {
+        virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
+                       _("Invalid snapshot_image_format '%1$s'"),
+                       snapstr);
+        return -1;
+    }
+
     if (virConfGetValueString(conf, "auto_dump_path", &cfg->autoDumpPath) < 0)
         return -1;
     if (virConfGetValueBool(conf, "auto_dump_bypass_cache", &cfg->autoDumpBypassCache) < 0)
         return -1;
     if (virConfGetValueBool(conf, "auto_start_bypass_cache", &cfg->autoStartBypassCache) < 0)
         return -1;
+    if (virConfGetValueUInt(conf, "auto_start_delay", &cfg->autoStartDelayMS) < 0)
+        return -1;
+    if (virConfGetValueString(conf, "auto_shutdown_try_save", &autoShutdownTrySave) < 0)
+        return -1;
+
+    if (autoShutdownTrySave != NULL) {
+        if ((autoShutdownVal =
+             virDomainDriverAutoShutdownScopeTypeFromString(autoShutdownTrySave)) < 0) {
+            virReportError(VIR_ERR_INVALID_ARG,
+                           _("unknown auto_shutdown_try_save '%1$s'"),
+                           autoShutdownTrySave);
+            return -1;
+        }
+        cfg->autoShutdownTrySave = autoShutdownVal;
+    }
+
+    if (cfg->autoShutdownTrySave == VIR_DOMAIN_DRIVER_AUTO_SHUTDOWN_SCOPE_ALL ||
+        cfg->autoShutdownTrySave == VIR_DOMAIN_DRIVER_AUTO_SHUTDOWN_SCOPE_TRANSIENT) {
+        virReportError(VIR_ERR_INVALID_ARG, "%s",
+                       _("managed save cannot be requested for transient domains"));
+        return -1;
+    }
+
+    if (virConfGetValueString(conf, "auto_shutdown_try_shutdown", &autoShutdownTryShutdown) < 0)
+        return -1;
+
+    if (autoShutdownTryShutdown != NULL) {
+        if ((autoShutdownVal =
+             virDomainDriverAutoShutdownScopeTypeFromString(autoShutdownTryShutdown)) < 0) {
+            virReportError(VIR_ERR_INVALID_ARG,
+                           _("unknown auto_shutdown_try_shutdown '%1$s'"),
+                           autoShutdownTryShutdown);
+            return -1;
+        }
+        cfg->autoShutdownTryShutdown = autoShutdownVal;
+    }
+
+    if (virConfGetValueString(conf, "auto_shutdown_poweroff", &autoShutdownPoweroff) < 0)
+        return -1;
+
+    if (autoShutdownPoweroff != NULL) {
+        if ((autoShutdownVal =
+             virDomainDriverAutoShutdownScopeTypeFromString(autoShutdownPoweroff)) < 0) {
+            virReportError(VIR_ERR_INVALID_ARG,
+                           _("unknown auto_shutdown_poweroff '%1$s'"),
+                           autoShutdownPoweroff);
+            return -1;
+        }
+        cfg->autoShutdownPoweroff = autoShutdownVal;
+    }
+
+    if (virConfGetValueUInt(conf, "auto_shutdown_wait",
+                            &cfg->autoShutdownWait) < 0)
+        return -1;
+    if (virConfGetValueBool(conf, "auto_shutdown_restore", &cfg->autoShutdownRestore) < 0)
+        return -1;
+    if (virConfGetValueBool(conf, "auto_save_bypass_cache",
+                            &cfg->autoSaveBypassCache) < 0)
+        return -1;
 
     return 0;
 }
@@ -681,6 +814,9 @@ virQEMUDriverConfigLoadProcessEntry(virQEMUDriverConfig *cfg,
     if (virConfGetValueString(conf, "dbus_daemon", &cfg->dbusDaemonName) < 0)
         return -1;
 
+    if (virConfGetValueString(conf, "qemu_rdp", &cfg->qemuRdpName) < 0)
+        return -1;
+
     if (virConfGetValueBool(conf, "set_process_name", &cfg->setProcessName) < 0)
         return -1;
     if (virConfGetValueUInt(conf, "max_processes", &cfg->maxProcesses) < 0)
@@ -1096,6 +1232,32 @@ virQEMUDriverConfigLoadStorageEntry(virQEMUDriverConfig *cfg,
 }
 
 
+static int
+virQEMUDriverConfigLoadFilesystemEntry(virQEMUDriverConfig *cfg,
+                                       virConf *conf)
+{
+    char **iter;
+
+    if (virConfGetValueStringList(conf, "shared_filesystems", false,
+                                  &cfg->sharedFilesystems) < 0)
+        return -1;
+
+    if (!cfg->sharedFilesystems)
+        return 0;
+
+    /* The paths provided by the user might contain trailing slashes
+     * and other fun diversions, which would break the naive string
+     * comparisons that we're later going to use them for */
+    for (iter = cfg->sharedFilesystems; *iter; iter++) {
+        char *canon = virFileCanonicalizePath(*iter);
+        g_free(*iter);
+        *iter = canon;
+    }
+
+    return 0;
+}
+
+
 int virQEMUDriverConfigLoadFile(virQEMUDriverConfig *cfg,
                                 const char *filename,
                                 bool privileged)
@@ -1125,6 +1287,9 @@ int virQEMUDriverConfigLoadFile(virQEMUDriverConfig *cfg,
     if (virQEMUDriverConfigLoadSPICEEntry(cfg, conf) < 0)
         return -1;
 
+    if (virQEMUDriverConfigLoadRDPEntry(cfg, conf) < 0)
+        return -1;
+
     if (virQEMUDriverConfigLoadSpecificTLSEntry(cfg, conf) < 0)
         return -1;
 
@@ -1170,6 +1335,9 @@ int virQEMUDriverConfigLoadFile(virQEMUDriverConfig *cfg,
     if (virQEMUDriverConfigLoadStorageEntry(cfg, conf) < 0)
         return -1;
 
+    if (virQEMUDriverConfigLoadFilesystemEntry(cfg, conf) < 0)
+        return -1;
+
     return 0;
 }
 
@@ -1209,6 +1377,14 @@ virQEMUDriverConfigValidate(virQEMUDriverConfig *cfg)
         return -1;
     }
 
+    if (cfg->rdpTLSx509certdir &&
+        !virFileExists(cfg->rdpTLSx509certdir)) {
+        virReportError(VIR_ERR_CONF_SYNTAX,
+                       _("rdp_tls_x509_cert_dir directory '%1$s' does not exist"),
+                       cfg->rdpTLSx509certdir);
+        return -1;
+    }
+
     if (cfg->chardevTLSx509certdir &&
         !virFileExists(cfg->chardevTLSx509certdir)) {
         virReportError(VIR_ERR_CONF_SYNTAX,
@@ -1233,14 +1409,6 @@ virQEMUDriverConfigValidate(virQEMUDriverConfig *cfg)
         return -1;
     }
 
-    if (cfg->vxhsTLSx509certdir &&
-        !virFileExists(cfg->vxhsTLSx509certdir)) {
-        virReportError(VIR_ERR_CONF_SYNTAX,
-                       _("vxhs_tls_x509_cert_dir directory '%1$s' does not exist"),
-                       cfg->vxhsTLSx509certdir);
-        return -1;
-    }
-
     if (cfg->nbdTLSx509certdir &&
         !virFileExists(cfg->nbdTLSx509certdir)) {
         virReportError(VIR_ERR_CONF_SYNTAX,
@@ -1269,7 +1437,6 @@ virQEMUDriverConfigSetDefaults(virQEMUDriverConfig *cfg)
     SET_TLS_SECRET_UUID_DEFAULT(chardev);
     SET_TLS_SECRET_UUID_DEFAULT(migrate);
     SET_TLS_SECRET_UUID_DEFAULT(backup);
-    SET_TLS_SECRET_UUID_DEFAULT(vxhs);
     SET_TLS_SECRET_UUID_DEFAULT(nbd);
 
 #undef SET_TLS_SECRET_UUID_DEFAULT
@@ -1294,10 +1461,10 @@ virQEMUDriverConfigSetDefaults(virQEMUDriverConfig *cfg)
 
     SET_TLS_X509_CERT_DEFAULT(vnc);
     SET_TLS_X509_CERT_DEFAULT(spice);
+    SET_TLS_X509_CERT_DEFAULT(rdp);
     SET_TLS_X509_CERT_DEFAULT(chardev);
     SET_TLS_X509_CERT_DEFAULT(migrate);
     SET_TLS_X509_CERT_DEFAULT(backup);
-    SET_TLS_X509_CERT_DEFAULT(vxhs);
     SET_TLS_X509_CERT_DEFAULT(nbd);
 
 #undef SET_TLS_X509_CERT_DEFAULT
@@ -1516,10 +1683,11 @@ virQEMUDriverGetDomainCapabilities(virQEMUDriver *driver,
     if (!(domCaps = virDomainCapsNew(path, machine, arch, virttype)))
         return NULL;
 
-    if (virQEMUCapsFillDomainCaps(qemuCaps, driver->hostarch,
-                                  domCaps, driver->privileged,
-                                  cfg->firmwares,
-                                  cfg->nfirmwares) < 0)
+    if (virQEMUCapsFillDomainCaps(cfg,
+                                  qemuCaps,
+                                  driver->hostarch,
+                                  domCaps,
+                                  driver->privileged) < 0)
         return NULL;
 
     return g_steal_pointer(&domCaps);
