@@ -202,7 +202,8 @@ qemuDomainDeviceDiskDefPostParseRestoreSecAlias(virDomainDiskDef *disk,
 
 int
 qemuDomainDeviceDiskDefPostParse(virDomainDiskDef *disk,
-                                 unsigned int parseFlags)
+                                 unsigned int parseFlags,
+                                 virQEMUCaps *qemuCaps)
 {
     virStorageSource *n;
 
@@ -220,6 +221,50 @@ qemuDomainDeviceDiskDefPostParse(virDomainDiskDef *disk,
         disk->mirror->format == VIR_STORAGE_FILE_NONE)
         disk->mirror->format = VIR_STORAGE_FILE_RAW;
 
+    /* default USB disk model:
+     *
+     * Historically we didn't use model for USB disks. It became necessary once
+     * it turned out that 'usb-storage' doesn't properly expose CDROM devices
+     * with -blockdev. Additionally 'usb-bot' which does properly handle them,
+     * while having identical implementation in qemu and driver in guest, are
+     * not in fact ABI compatible. Thus the logic is as follows:
+     *
+     * If ABI update is not allowed:
+     *  - use 'usb-storage' for either (unless only 'usb-bot' is supported)
+     *
+     * If ABI update is possible
+     * - for VIR_DOMAIN_DISK_DEVICE_DISK use 'usb-storage' as it doesn't matter
+     *    (it is identical with 'usb-bot' ABI wise)
+     * - for VIR_DOMAIN_DISK_DEVICE_CDROM use 'usb-bot' if available
+     *    (as it properly exposes cdrom)
+     *
+     * When formatting migratable XML the code strips 'usb-storage' to preserve
+     * migration to older daemons. If a new definition with 'usb-bot' cdrom is
+     * created via new start or hotplug it will fail migrating. Users must
+     * explicitly set the broken config in XML or unplug the device.
+     */
+    if (qemuCaps &&
+        disk->bus == VIR_DOMAIN_DISK_BUS_USB &&
+        disk->model == VIR_DOMAIN_DISK_MODEL_DEFAULT) {
+
+        if (disk->device == VIR_DOMAIN_DISK_DEVICE_CDROM &&
+            parseFlags & VIR_DOMAIN_DEF_PARSE_ABI_UPDATE) {
+
+            if (virQEMUCapsGet(qemuCaps, QEMU_CAPS_DEVICE_USB_BOT)) {
+                disk->model = VIR_DOMAIN_DISK_MODEL_USB_BOT;
+            } else if (virQEMUCapsGet(qemuCaps, QEMU_CAPS_DEVICE_USB_STORAGE)) {
+                disk->model = VIR_DOMAIN_DISK_MODEL_USB_STORAGE;
+            }
+
+        } else {
+            if (virQEMUCapsGet(qemuCaps, QEMU_CAPS_DEVICE_USB_STORAGE)) {
+                disk->model = VIR_DOMAIN_DISK_MODEL_USB_STORAGE;
+            } else if (virQEMUCapsGet(qemuCaps, QEMU_CAPS_DEVICE_USB_BOT)) {
+                disk->model = VIR_DOMAIN_DISK_MODEL_USB_BOT;
+            }
+        }
+    }
+
     /* default disk encryption engine */
     for (n = disk->src; virStorageSourceIsBacking(n); n = n->backingStore) {
         if (n->encryption && n->encryption->engine == VIR_STORAGE_ENCRYPTION_ENGINE_DEFAULT)
@@ -429,6 +474,7 @@ qemuDomainControllerDefPostParse(virDomainControllerDef *cont,
     case VIR_DOMAIN_CONTROLLER_TYPE_FDC:
     case VIR_DOMAIN_CONTROLLER_TYPE_XENBUS:
     case VIR_DOMAIN_CONTROLLER_TYPE_ISA:
+    case VIR_DOMAIN_CONTROLLER_TYPE_NVME:
     case VIR_DOMAIN_CONTROLLER_TYPE_LAST:
         break;
     }
@@ -842,7 +888,7 @@ qemuDomainDeviceDefPostParse(virDomainDeviceDef *dev,
         break;
 
     case VIR_DOMAIN_DEVICE_DISK:
-        ret = qemuDomainDeviceDiskDefPostParse(dev->data.disk, parseFlags);
+        ret = qemuDomainDeviceDiskDefPostParse(dev->data.disk, parseFlags, qemuCaps);
         break;
 
     case VIR_DOMAIN_DEVICE_VIDEO:
@@ -1090,24 +1136,15 @@ qemuDomainDefBootPostParse(virDomainDef *def,
 }
 
 
-static int
+static void
 qemuDomainDefAddImplicitInputDevice(virDomainDef *def,
                                     virQEMUCaps *qemuCaps)
 {
     if (virQEMUCapsSupportsI8042(qemuCaps, def) &&
         def->features[VIR_DOMAIN_FEATURE_PS2] != VIR_TRISTATE_SWITCH_OFF) {
-        if (virDomainDefMaybeAddInput(def,
-                                      VIR_DOMAIN_INPUT_TYPE_MOUSE,
-                                      VIR_DOMAIN_INPUT_BUS_PS2) < 0)
-            return -1;
-
-        if (virDomainDefMaybeAddInput(def,
-                                      VIR_DOMAIN_INPUT_TYPE_KBD,
-                                      VIR_DOMAIN_INPUT_BUS_PS2) < 0)
-            return -1;
+        virDomainDefMaybeAddInput(def, VIR_DOMAIN_INPUT_TYPE_MOUSE, VIR_DOMAIN_INPUT_BUS_PS2);
+        virDomainDefMaybeAddInput(def, VIR_DOMAIN_INPUT_TYPE_KBD, VIR_DOMAIN_INPUT_BUS_PS2);
     }
-
-    return 0;
 }
 
 
@@ -1202,8 +1239,7 @@ qemuDomainDefAddDefaultDevices(virQEMUDriver *driver,
     bool addIOMMU = false;
 
     /* add implicit input devices */
-    if (qemuDomainDefAddImplicitInputDevice(def, qemuCaps) < 0)
-        return -1;
+    qemuDomainDefAddImplicitInputDevice(def, qemuCaps);
 
     /* Add implicit PCI root controller if the machine has one */
     switch (def->os.arch) {
@@ -1342,15 +1378,11 @@ qemuDomainDefAddDefaultDevices(virQEMUDriver *driver,
         break;
     }
 
-    if (addDefaultUSB &&
-        virDomainControllerFind(def, VIR_DOMAIN_CONTROLLER_TYPE_USB, 0) < 0 &&
-        virDomainDefAddUSBController(def, 0, usbModel) < 0)
-        return -1;
+    if (addDefaultUSB && virDomainControllerFind(def, VIR_DOMAIN_CONTROLLER_TYPE_USB, 0) < 0)
+        virDomainDefAddUSBController(def, 0, usbModel);
 
-    if (addImplicitSATA &&
-        virDomainDefMaybeAddController(
-            def, VIR_DOMAIN_CONTROLLER_TYPE_SATA, 0, -1) < 0)
-        return -1;
+    if (addImplicitSATA)
+        virDomainDefMaybeAddController(def, VIR_DOMAIN_CONTROLLER_TYPE_SATA, 0, -1);
 
     pciRoot = virDomainControllerFind(def, VIR_DOMAIN_CONTROLLER_TYPE_PCI, 0);
 
@@ -1365,9 +1397,9 @@ qemuDomainDefAddDefaultDevices(virQEMUDriver *driver,
                                virDomainControllerModelPCITypeToString(def->controllers[pciRoot]->model));
                 return -1;
             }
-        } else if (!virDomainDefAddController(def, VIR_DOMAIN_CONTROLLER_TYPE_PCI, 0,
-                                              VIR_DOMAIN_CONTROLLER_MODEL_PCI_ROOT)) {
-            return -1;
+        } else {
+            virDomainDefAddController(def, VIR_DOMAIN_CONTROLLER_TYPE_PCI, 0,
+                                      VIR_DOMAIN_CONTROLLER_MODEL_PCI_ROOT);
         }
     }
 
@@ -1386,9 +1418,9 @@ qemuDomainDefAddDefaultDevices(virQEMUDriver *driver,
                                virDomainControllerModelPCITypeToString(def->controllers[pciRoot]->model));
                 return -1;
             }
-        } else if (!virDomainDefAddController(def, VIR_DOMAIN_CONTROLLER_TYPE_PCI, 0,
-                                             VIR_DOMAIN_CONTROLLER_MODEL_PCIE_ROOT)) {
-            return -1;
+        } else {
+            virDomainDefAddController(def, VIR_DOMAIN_CONTROLLER_TYPE_PCI, 0,
+                                      VIR_DOMAIN_CONTROLLER_MODEL_PCIE_ROOT);
         }
     }
 
@@ -1420,19 +1452,11 @@ qemuDomainDefAddDefaultDevices(virQEMUDriver *driver,
             addDefaultUSBMouse = false;
     }
 
-    if (addDefaultUSBKBD &&
-        def->ngraphics > 0 &&
-        virDomainDefMaybeAddInput(def,
-                                  VIR_DOMAIN_INPUT_TYPE_KBD,
-                                  VIR_DOMAIN_INPUT_BUS_USB) < 0)
-        return -1;
+    if (addDefaultUSBKBD && def->ngraphics > 0)
+        virDomainDefMaybeAddInput(def, VIR_DOMAIN_INPUT_TYPE_KBD, VIR_DOMAIN_INPUT_BUS_USB);
 
-    if (addDefaultUSBMouse &&
-        def->ngraphics > 0 &&
-        virDomainDefMaybeAddInput(def,
-                                  VIR_DOMAIN_INPUT_TYPE_MOUSE,
-                                  VIR_DOMAIN_INPUT_BUS_USB) < 0)
-        return -1;
+    if (addDefaultUSBMouse && def->ngraphics > 0)
+        virDomainDefMaybeAddInput(def, VIR_DOMAIN_INPUT_TYPE_MOUSE, VIR_DOMAIN_INPUT_BUS_USB);
 
     if (addPanicDevice) {
         virDomainPanicModel defaultModel = qemuDomainDefaultPanicModel(def);
